import { Module } from 'module';
import { createContext, runInContext } from 'vm';
import { transformSync } from 'esbuild';

const checkArg = (arg) => {
    const argType = typeof arg;
    if (argType !== 'string') {
        throw new Error(`Argument must be string, not '${argType}'.`);
    }
};

const requireFromString = (code, globals = {}) => {
    checkArg(code);
    const _module = new Module(String(new Date().valueOf()));
    const context = createContext(Object.assign({ __dirname,
        __filename, exports: _module.exports, module: _module, require }, globals));
    runInContext(code, context);
    return context.module.exports;
};

const importFromString = ({ code, transformOptions, globals = {} }) => {
    checkArg(code);
    const { code: transformedCode } = transformSync(code, Object.assign({ format: 'cjs' }, transformOptions));
    return requireFromString(transformedCode, globals);
};

export { importFromString, requireFromString };
