'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var module$1 = require('module');
var vm = require('vm');
var esbuild = require('esbuild');

const checkArg = (arg) => {
    const argType = typeof arg;
    if (argType !== 'string') {
        throw new Error(`Argument must be string, not '${argType}'.`);
    }
};

const requireFromString = (code, globals = {}) => {
    checkArg(code);
    const _module = new module$1.Module(String(new Date().valueOf()));
    const context = vm.createContext(Object.assign({ __dirname,
        __filename, exports: _module.exports, module: _module, require }, globals));
    vm.runInContext(code, context);
    return context.module.exports;
};

const importFromString = ({ code, transformOptions, globals = {} }) => {
    checkArg(code);
    const { code: transformedCode } = esbuild.transformSync(code, Object.assign({ format: 'cjs' }, transformOptions));
    return requireFromString(transformedCode, globals);
};

exports.importFromString = importFromString;
exports.requireFromString = requireFromString;
